import { Point } from "../spatial/Point.js";

export class Character {
    constructor(atlas, x = 0, y = 0) {
        this.atlas = atlas
        this.position = new Point(x * atlas.tileDimensions.width,y * atlas.tileDimensions.height)
        this.gridPosition = new Point(x, y)
    }

    load() {
        let characterData = JSON.parse(localStorage.getItem('teetopia-player'))
        if(characterData) {
            this.body = this.atlas.getDrawableSprite(characterData.bodySprite[0], characterData.bodySprite[1])
            this.clothingTop = this.atlas.getDrawableSprite(characterData.topSprite[0], characterData.topSprite[1])
            this.clothingBottom = this.atlas.getDrawableSprite(characterData.bottomSprite[0], characterData.bottomSprite[1])
            this.headHair = this.atlas.getDrawableSprite(characterData.hairSprite[0], characterData.hairSprite[1])
            this.room = characterData.room ?? "room5"
        }
    }

    save() {
        let characterData = JSON.parse(localStorage.getItem('teetopia-player'))
        if(characterData) {
            characterData["bodySprite"] = this.body.getGridPosition()
            characterData["topSprite"] = this.clothingTop.getGridPosition()
            characterData["bottomSprite"] = this.clothingBottom.getGridPosition()
            characterData["hairSprite"] = this.headHair.getGridPosition()
            characterData["room"] = this.room
            characterData["position"] = `${this.position.x},${this.position.y}`
        }
        localStorage['teetopia-player'] = JSON.stringify(characterData)
    }

    update(delta) {

    }

    draw(ctx, scaledCanvas) {
        ctx.save()
        ctx.translate(this.position.x, this.position.y)
        this.body.draw(ctx, scaledCanvas)
        this.clothingBottom.draw(ctx, scaledCanvas)
        this.clothingTop.draw(ctx, scaledCanvas)
        this.headHair.draw(ctx, scaledCanvas)
        ctx.restore()
    }
}
