using System;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Configuration;

using gameapi.Library;
using System.Text.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Dynamic;

namespace gameapi.Controllers
{
	[ApiController]
	[Route("api/room")]
	[Produces("application/json")]
	public class RoomController : ControllerBase
	{
		private readonly ILogger<WorldController> logger;
		private readonly IConfiguration configuration;

		private JsonFileReader jsonFileReader;
		private JsonFileSaver jsonFileSaver;

		public RoomController(IConfiguration config, ILogger<WorldController> log)
		{
			configuration = config;
			logger = log;
			this.jsonFileReader = new JsonFileReader();
			this.jsonFileSaver = new JsonFileSaver();
		}

		[HttpGet("{roomName}")]
		public IActionResult Get(string roomName)
		{
			JsonElement response = new JsonElement();
			try
			{
				response = this.jsonFileReader.ReadGameData($"rooms/{roomName}.json");
			}
			catch (System.IO.FileNotFoundException)
			{
				Console.WriteLine($"Unable to find room file: {roomName}");
				try
				{
					response = this.jsonFileReader.Read($"../client/maps/{roomName}.json");
				}
				catch (System.IO.FileNotFoundException)
				{
					dynamic invalidSession = new
					{
						responseType = StatusCodes.Status400BadRequest,
						responseMessage = $"Room not found."
					};
					return StatusCode(StatusCodes.Status400BadRequest, invalidSession);
				}
			}

			//TODO: version control progress
			return Ok(response);
		}

		[HttpPost("{roomName}/save")]
		public IActionResult SaveRoom(string roomName, [FromBody] dynamic body)
		{
			var tokenProp = body.GetProperty("token");
			var token = tokenProp.ToString();
			var id = "";
			var email = "";
			try
			{
				var sessionData = new SessionRetriever().GetValid(token);
				var idProp = sessionData.GetProperty("id");
				id = idProp.ToString();
				var emailProp = sessionData.GetProperty("email");
				email = emailProp.ToString();
			}
			catch (Exception)
			{
				dynamic invalidSession = new
				{
					responseType = StatusCodes.Status400BadRequest,
					responseMessage = $"Invalid session token. Please log in!"
				};
				return StatusCode(StatusCodes.Status400BadRequest, invalidSession);
			}
			//check existing room exists
			try
			{
				var roomData = this.jsonFileReader.ReadGameData($"rooms/{roomName}.json");
				//check user has permissions to existing room
				var ownerProp = roomData.GetProperty("owner");
				var owner = ownerProp.ToString();
				var writers = new List<string>();
				foreach (JsonElement writer in roomData.GetProperty("writers").EnumerateArray())
				{
					writers.Add(writer.ToString());
				}
				writers.Add(owner);

				var allowedId = writers.Find(x => x == id);
				if (allowedId == null && owner != "")
				{
					dynamic userNoPermissionsResponse = new
					{
						responseType = StatusCodes.Status400BadRequest,
						responseMessage = $"Logged in user {email}:{id} does not have permissions to save room {roomName}."
					};
					return StatusCode(StatusCodes.Status400BadRequest, userNoPermissionsResponse);
				}
				//overwrite existing room
				this.jsonFileSaver.SaveGameData($"rooms/{roomName}.json", body.GetProperty("roomData"));
				dynamic response = new
				{
					responseType = StatusCodes.Status200OK,
					responseMessage = "Save successful!"
				};
				return StatusCode(200, response);
			}
			catch (System.IO.FileNotFoundException)
			{
				JsonElement roomData = body.GetProperty("roomData");

				this.jsonFileSaver.SaveGameData($"rooms/{roomName}.json", roomData);
				dynamic response = new
				{
					responseType = StatusCodes.Status200OK,
					responseMessage = "New room save successful!"
				};
				return StatusCode(200, response);
			}
		}

		private void validateSessionDate(JsonElement sessionData) {
			var dateProp = sessionData.GetProperty("validUntil").ToString();
			var date = DateTime.Parse(dateProp.ToString());
			if(DateTime.Now > date) {
				throw new Exception("Session expired.");
			}
		}
	}
}
